@@ -1,6 +1,6 @@
 package Bundle::CPAN;
 
-$VERSION = '1.58';
+$VERSION = '1.81';
 
 1;
 
@@ -16,9 +16,13 @@ C<perl -MCPAN -e 'install Bundle::CPAN'>
 
 =head1 CONTENTS
 
-File::Spec
+Digest::SHA
 
-Digest::MD5
+Module::Signature
+
+File::Temp
+
+File::Spec
 
 Compress::Zlib
 
@@ -30,22 +34,21 @@ Term::ReadKey
 
 Term::ReadLine::Perl # sorry, I'm discriminating the ::Gnu module
 
-CPAN::WAIT
+YAML
+
+Text::Glob
+
+Module::Build
 
 CPAN
 
 =head1 DESCRIPTION
 
-This bundle includes CPAN.pm as the base module and CPAN::WAIT, a
-plugin for the first WAIT based CPAN search engine.
+This bundle includes CPAN.pm as the base module.
 
 When CPAN installs this bundle it tries immediately to enable
 Term::ReadLine so that you do not need to restart your CPAN session.
 
-Compress::Zlib needs as a prerequisite the zlib library. Currently
-(January 1998) this library is not shipped with the Compress::Zlib
-distribution.
-
 In this bundle Term::ReadLine::Perl is preferred over
 Term::ReadLine::Gnu because I expect that it gives less problems on
 portability.
@@ -53,8 +56,9 @@ portability.
 Note that all modules in this Bundle are not strict prerequisites to
 get a working CPAN.pm. CPAN.pm can work quite well without the other
 modules (except for Net::FTP which is really highly recommended). The
-other modules are just goodies that make using CPAN.pm more fun.
+other modules are B<suggested> and can safely be installed later or
+not at all.
 
 =head1 AUTHOR
 
-Andreas Knig
+Andreas Koenig
